/// Class representing current weather forecast value.
class CityWeather {
  Coordinate coord;
  List<Weather> weather;
  String base;
  MainWeatherData main;
  int visibility;
  Wind wind;
  Clouds clouds;
  Rain rain;
  Snow snow;
  int dt;
  Sys sys;
  int timezone;
  int id;
  String name;
  int cod;
  String dtTxt;

  DateTime get timeStamp =>
      DateTime.fromMillisecondsSinceEpoch(this.dt * 1000).toUtc();

  CityWeather(
      {this.coord,
      this.weather,
      this.base,
      this.main,
      this.visibility,
      this.wind,
      this.clouds,
      this.dt,
      this.sys,
      this.timezone,
      this.id,
      this.name,
      this.cod,
      this.rain,
      this.snow,
      this.dtTxt});

  CityWeather.fromJSON(Map<String, dynamic> data)
      : coord = (data['coord'] != null)
            ? new Coordinate.fromJSON(data['coord'])
            : null,
        weather = List<Weather>.from(
            data['weather'].map((w) => new Weather.fromJSON(w))),
        base = data['base'],
        main = new MainWeatherData.fromJSON(data['main']),
        visibility = data['visibility'],
        snow = (data['snow'] != null) ? new Snow.fromJSON(data['snow']) : null,
        rain = (data['rain'] != null) ? new Rain.fromJSON(data['rain']) : null,
        wind = (data['wind'] != null) ? new Wind.fromJSON(data['wind']) : null,
        clouds = (data['clouds'] != null)
            ? new Clouds.fromJSON(data['clouds'])
            : null,
        dt = data['dt'],
        sys = (data['sys'] != null) ? new Sys.fromJSON(data['sys']) : null,
        timezone = data['timezone'],
        id = data['id'],
        name = data['name'],
        cod = (data['cod'] is String) ? int.parse(data['cod']) : data['cod'],
        dtTxt = data['dtTxt'];
}

///Rain Data
class Rain {
  double threeH;
  double oneH;

  Rain({this.threeH, this.oneH});

  Rain.fromJSON(Map<String, dynamic> data)
      : threeH = data['3h']?.toDouble(),
        oneH = data['1h']?.toDouble();
}

/// Snow Data
class Snow {
  double threeH;
  double oneH;
  Snow({this.threeH, this.oneH});

  Snow.fromJSON(Map<String, dynamic> data)
      : threeH = data['3h']?.toDouble(),
        oneH = data['1h']?.toDouble();
}

/// Class representing clould data found in current weather forecast.
class Clouds {
  int all;
  int today;

  Clouds({this.all, this.today});
  Clouds.fromJSON(Map<String, dynamic> data)
      : all = data['all'],
        today = data['today'];
}

/// Simple coordinate data.
class Coordinate {
  double longitude;
  double latitude;

  double get lat => this.latitude;
  double get lon => this.longitude;

  Coordinate({
    this.longitude,
    this.latitude,
  });

  Coordinate.fromJSON(Map<String, dynamic> data)
      : longitude = data['lon'],
        latitude = data['lat'];
}

/// Main weather data in numbers instead of pretty human readable form.
class MainWeatherData {
  double temperature;
  double pressure;
  int humidity;
  double temperatureMin;
  double temperatureMax;
  double seaLevel;
  double groundLevel;
  double tempKF;

  double get temp => this.temperature;
  double get tempMax => this.temperatureMax;
  double get tempMin => this.temperatureMin;

  MainWeatherData(
      {this.temperature,
      this.pressure,
      this.humidity,
      this.temperatureMin,
      this.temperatureMax,
      this.seaLevel,
      this.groundLevel,
      this.tempKF});

  MainWeatherData.fromJSON(Map<String, dynamic> data)
      : temperature = data['temp'].toDouble(),
        pressure = data['pressure'].toDouble(),
        humidity = data['humidity'],
        temperatureMax = data['temp_max'].toDouble(),
        temperatureMin = data['temp_min'].toDouble(),
        seaLevel = data['sea_level']?.toDouble(),
        groundLevel = data['grnd_level']?.toDouble(),
        tempKF = data['temp_kf']?.toDouble();
}

/// 'Sys' data, don't know the full form.
class Sys {
  int type;
  int id;
  double message;
  String country;
  int sunrise;
  int sunset;
  String pod;

  Sys(
      {this.type,
      this.id,
      this.message,
      this.country,
      this.sunrise,
      this.sunset,
      this.pod});

  Sys.fromJSON(Map<String, dynamic> data)
      : type = data['type'],
        id = data['id'],
        message = data['message']?.toDouble(),
        country = data['country'],
        sunrise = data['sunrise'],
        sunset = data['sunset'],
        pod = data['pod'];
}

/// Human readable weather data.
class Weather {
  int id;
  String main;
  String description;
  String icon;

  Weather({
    this.id,
    this.main,
    this.description,
    this.icon,
  });

  Weather.fromJSON(Map<String, dynamic> data)
      : id = data['id'],
        main = data['main'],
        description = data['description'],
        icon = data['icon'];
}

/// Wind data details.
class Wind {
  double speed;
  double degree;

  double get deg => this.degree;

  Wind({
    this.speed,
    this.degree,
  });

  Wind.fromJSON(Map<String, dynamic> data)
      : speed = data['speed'].toDouble(),
        degree = data['deg'].toDouble();
}

/// A class representing response returned for the query to get all weather
/// data from a rectangular geolocation.
class RectLocResponse {
  int cod;
  double calculationTime;
  int cnt;
  List<CityWeather> list;

  double get calctime => this.calculationTime;

  RectLocResponse({this.cod, this.calculationTime, this.cnt, this.list});

  RectLocResponse.fromJSON(Map<String, dynamic> data)
      : cod = (data['cod'] is String) ? int.parse(data['cod']) : data['cod'],
        calculationTime = data['calctime'],
        cnt = data['cnt'],
        list = List<CityWeather>.from(
            data['list'].map((w) => new CityWeather.fromJSON(w)));
}

///A class representing response returned from the query to get all weather
/// data from a circle of area
class CitiesInCycleResponse {
  String message;
  int cod;
  int count;
  List<CityWeather> list;

  CitiesInCycleResponse({this.cod, this.message, this.count, this.list});

  CitiesInCycleResponse.fromJSON(Map<String, dynamic> data)
      : cod = (data['cod'] is String) ? int.parse(data['cod']) : data['cod'],
        count = data['count'],
        message = data['message'],
        list = List<CityWeather>.from(
            data['list'].map((w) => new CityWeather.fromJSON(w)));
}

/// Class representing response for bulk data request for cities by id
class CitiesByIDResponse {
  int cnt;
  List<CityWeather> list;

  CitiesByIDResponse({this.cnt, this.list});

  CitiesByIDResponse.fromJSON(Map<String, dynamic> data)
      : cnt = data['cnt'],
        list = List<CityWeather>.from(
            data['list'].map((w) => new CityWeather.fromJSON(w)));
}

/// Hourly Forecast
class ForecastResponse {
  double message;
  int cod;
  int cnt;
  List<CityWeather> list;

  ForecastResponse({this.cod, this.message, this.cnt, this.list});

  ForecastResponse.fromJSON(Map<String, dynamic> data)
      : cod = (data['cod'] is String) ? int.parse(data['cod']) : data['cod'],
        cnt = data['cnt'],
        message = (data['message'] is String)
            ? double.parse(data['message'])
            : data['message'],
        list = List<CityWeather>.from(
            data['list'].map((w) => new CityWeather.fromJSON(w)));
}

/// UV index data class
class UVIndex {
  double longitude;
  double latitude;
  int date;
  String dateISO;
  double value;

  double get lat => this.latitude;
  double get lon => this.longitude;
  DateTime get timeStamp =>
      DateTime.fromMillisecondsSinceEpoch(this.date * 1000).toUtc();

  UVIndex({this.longitude, this.latitude, this.date, this.dateISO, this.value});

  UVIndex.fromJSON(Map<String, dynamic> data)
      : longitude = data['lon'],
        latitude = data['lat'],
        date = data['date'],
        dateISO = data['date_iso'],
        value = (data['value'] is String)
            ? double.parse(data['value'])
            : data['value'];
}
